const wgtVer = '1.3'; //当前版本号
const actions = `https://www.freeart.cc/home/version`; //请求服务器版本资源路径
let newVer = null; //最新版本号
let ajaxCount = 0; //设置初始请求次数，请求更新的接口有可能会失败 导致没法强制更新；
let dtask = null; //下载任务

//plus环境准备就绪
mui.plusReady(() => {
  //设置顶部栏样式
  setBarColor();

  //处理页面返回
  handleBack();

  //处理监听
  handleListener();

  //检查更新
  checkUpdate();
});

//设置顶部栏样式
const setBarColor = () => {
  plus.navigator.setStatusBarBackground('#1A1A1A');
  plus.navigator.setStatusBarStyle('light');
};

//检查更新
const checkUpdate = () => {
  mui.ajax(actions, {
    data: { type: mui.os.android ? 'android' : 'ios' },
    dataType: 'json', //服务器返回json格式数据
    headers: {
      locale: localStorage.getItem('locale') || 'CN',
    },
    type: 'get', //HTTP请求类型
    timeout: 10000, //超时时间设置为10秒
    success: (data) => {
      ajaxCount = 0; //接口成功后初始化计数
      if (data.code == 200) {
        newVer = data.data[0].code; //服务器版本号
        let downloadUrl = data.data[0].url; //服务器新包地址 安卓是apk地址  ios 是下载页地址
        if (wgtVer && newVer && wgtVer != newVer) {
          updateDialog(downloadUrl, newVer);
        }
      }
    },
    error: (xhr, type, errorThrown) => {
      console.log(type);
      //如果接口报异常 2s后再次请求 直到请求正常  设置最大请求次数  ajaxCount<5
      if (ajaxCount < 5) {
        setTimeout(() => {
          ajaxCount++;
          checkUpdate();
        }, 2000);
      }
    },
  });
};

//是否更新提示框
const updateDialog = (downloadUrl, newVer) => {
  //弹出更新提示
  var btnArray = ['确认', '取消'];
  mui.confirm(`是否下载最新版本${newVer}安装包?`, '', btnArray, function (e) {
    if (e.index == 0) {
      //加入进度条节点
      let progressHtml =
        "<div class='plus_download_progress'> <div class='plus_download_progress_box'> <h4>Downloading (0%)</h4> <progress value='0'></progress> </div> </div>";
      document
        .getElementById('app')
        .insertAdjacentHTML('afterend', progressHtml);
      createDownloadTask(downloadUrl);
    } else {
      mui.os.android ? plus.runtime.quit() : plus.runtime.restart();
    }
  });
};

//创建下载任务
const createDownloadTask = (downloadUrl) => {
  if (mui.os.android) {
    dtask = plus.downloader.createDownload(downloadUrl, {}, (d, status) => {
      if (status == 200) {
        installPackage(d.filename);
        plus.runtime.quit();
      } else {
        plus.nativeUI.alert('未能下载更新包!', function () {}, '', 'ok');
      }
    });

    dtask.addEventListener('statechanged', onStateChanged, false);
    dtask.start();
  } else {
    plus.runtime.openURL(downloadUrl);
    plus.runtime.restart();
  }
};

// 监听下载任务状态  展示下载进度条进度
const onStateChanged = () => {
  mui.later(function () {
    let num = dtask.downloadedSize / dtask.totalSize;
    if (num && num < 1) {
      mui('.plus_download_progress_box progress')[0].value = num;
      mui(
        '.plus_download_progress_box h4'
      )[0].innerText = `Downloading (${Math.floor(num * 100)}%)`;
    }
  }, 200);
};

//安装更新文件
const installPackage = (path) => {
  plus.runtime.install(
    path,
    {},
    function () {
      plus.nativeUI.closeWaiting();
      console.log('安装wgt文件成功！');
      plus.nativeUI.alert(
        '请在安装后重新打开应用程序！',
        function () {
          plus.runtime.restart();
        },
        '',
        'ok'
      );
    },
    function (e) {
      plus.nativeUI.closeWaiting();
      console.log('安装wgt文件失败[' + e.code + ']：' + e.message);
      plus.nativeUI.alert(
        'Failed to update resource [' + e.code + ']：' + e.message,
        function () {},
        '',
        'ok'
      );
    }
  );
};

// 处理document 监听事件
const handleListener = () => {
  let timeObj = null;
  //监听网络状态
  document.addEventListener(
    'netchange',
    () => {
      var nt = plus.networkinfo.getCurrentType();
      //获取历史状态
      var lastNt = sessionStorage.getItem('lastNt') || nt;
      timeObj && window.clearTimeout(timeObj);
      //加延时是为了避免无线wifi切数据时会插入一个无网络状态所造成的判断错误
      timeObj = setTimeout(() => {
        //更新历史状态
        sessionStorage.setItem('lastNt', nt);
        //网络状态为 0 或 1 时无网络服务
        if (nt == 1 || nt == 0) {
          mui.toast('目前没有网络服务，请检查网络连接！', '');
        } else if (lastNt == 1 || lastNt == 0) {
          //历史状态为 0 或 1 时 表示网络状态由无到有
          console.log('重新连接网络');
          window.location.reload();
        }
      }, 3000);
    },
    false
  );

  //监听app进入前台
  document.addEventListener(
    'resume',
    () => {
      //如果没有下载安装包任务 版本号也不一致 就再次请求接口确认
      if (!dtask && wgtVer != newVer) {
        checkUpdate();
      }
    },
    false
  );
};

//处理页面返回
const handleBack = () => {
  var quitTime = null;
  mui.back = function () {
    //首次按键，提示‘再按一次退出应用’
    if (!quitTime) {
      //记录第一次按下回退键的时间
      quitTime = new Date().getTime();
      //回退到上一页面
      window.history.go(-1);
      setTimeout(function () {
        //1s中后清除
        quitTime = null;
      }, 1000);
    } else {
      if (new Date().getTime() - quitTime < 1000) {
        quitTime = null;
        plus.runtime.quit();
      }
    }
  };
};
